"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import glob
import os
import shutil
import numpy as np
from transphire import transphire_utils as tu
from transphire import transphire_import as ti


def get_ctf_command(file_input, new_name, settings, queue_com, name):
    """
    Create the ctf command based on the ctf software.

    file_input - Input name of the file for ctf estimation
    new_name - Output file
    settings - TranSPHIRE settings
    queue_com - Queue for communication
    name - Name of process

    Returns:
    CTF command
    File to check vor validation if the process was successful
    """
    ctf_name = settings['Copy']['CTF']
    if ctf_name == 'CTFFIND4 v4.1.8' or \
            ctf_name == 'CTFFIND4 v4.1.10':
        command = create_ctffind_4_v4_1_8_command(
            ctf_name = ctf_name,
            file_input=file_input,
            file_output=new_name,
            settings=settings
            )
        check_files = []

    elif ctf_name == 'Gctf v1.06' or \
            ctf_name == 'Gctf v1.18':
        command = create_gctf_v1_06_command(
            ctf_name=ctf_name,
            file_input=file_input,
            file_output=new_name,
            settings=settings
            )
        check_files = ['{0}_gctf.star'.format(new_name)]

    elif ctf_name == 'CTER v1.0':
        output_dir, _ = os.path.splitext(new_name)
        command = create_cter_1_0_command(
            ctf_name=ctf_name,
            file_input=file_input,
            output_dir=output_dir,
            settings=settings
            )
        check_files = ['{0}/partres.txt'.format(output_dir)]

    else:
        message = '\n'.join([
            '{0}: Not known!'.format(settings['Copy']['CTF']),
            'Please contact the TranSPHIRE authors!'
            ])
        queue_com['error'].put(
            message,
            name
            )
        IOError(message)

    return command, check_files


def find_logfiles(root_path, file_name, settings, queue_com, name):
    """
    Find logfiles related to the produced CTF files.

    root_path - Root path of the file
    file_name - File name of the ctf file.
    settings - TranSPHIRE settings
    queue_com - Queue for communication
    name - Name of process

    Returns:
    list of log files
    """
    log_files = None
    copied_log_files = None
    ctf_root_path = os.path.join(settings['ctf_folder'], file_name)
    if settings['Copy']['CTF'] == 'CTFFIND4 v4.1.8' or \
            settings['Copy']['CTF'] == 'CTFFIND4 v4.1.10' or \
            settings['Copy']['CTF'] == 'CTER v1.0':
        copied_log_files = []
        recursive_file_search(directory=ctf_root_path, files=copied_log_files)
        log_files = copied_log_files

    elif settings['Copy']['CTF'] == 'Gctf v1.18' or \
            settings['Copy']['CTF'] == 'Gctf v1.06':
        log_files = []
        copied_log_files = []
        exclude_list = [
            '{0}.mrc'.format(root_path),
            '{0}.log'.format(root_path),
            '{0}.err'.format(root_path),
            ]
        for log_file in glob.glob('{0}*'.format(root_path)):
            if log_file in exclude_list:
                continue
            else:
                pass

            new_file = os.path.join(
                settings['ctf_folder'],
                os.path.basename(log_file)
                )
            log_files.append(log_file)
            copied_log_files.append(new_file)

    else:
        message = '\n'.join([
            '{0}: Not known!'.format(settings['Copy']['CTF']),
            'Please contact the TranSPHIRE authors!'
            ])
        queue_com['error'].put(
            message,
            name
            )
        raise IOError(message)

    assert log_files is not None
    assert copied_log_files is not None
    return log_files, copied_log_files


def recursive_file_search(directory, files):
    """
    Recursive file search function.
    """
    file_names = glob.glob('{0}*'.format(directory))
    for name in file_names:
        if os.path.isdir(name):
            recursive_file_search('{0}/'.format(name), files)
        else:
            files.append(name)


def create_gctf_v1_06_command(
        ctf_name, file_input, file_output, settings
        ):
    """Create the Gctf v1.06 command"""

    command = []
    # Start the program
    command.append('{0}'.format(settings['Path'][ctf_name]))
    ignore_list = []
    if settings[ctf_name]['Phase plate'] == 'False':
        ignore_list.append('Phase plate')
        ignore_list.append('--phase_shift_L')
        ignore_list.append('--phase_shift_H')
        ignore_list.append('--phase_shift_S')
        ignore_list.append('--phase_shift_T')
        ignore_list.append('--only_search_ps')
        ignore_list.append('--cosearch_refine_ps')
    else:
        ignore_list.append('Phase plate')
    if settings[ctf_name]['Use movies'] == 'False':
        ignore_list.append('Use movies')
        ignore_list.append('--do_mdef_refine')
        ignore_list.append('--mdef_ave_type')
        ignore_list.append('--mdef_aveN')
        ignore_list.append('--mdef_fit')
    else:
        ignore_list.append('Use movies')

    command.append('--ctfstar')
    command.append('{0}_gctf.star'.format(file_output))

    for key in settings[ctf_name]:
        if key in ignore_list:
            continue
        elif settings[ctf_name][key]:
            command.append(key)
            command.append(
                '{0}'.format(settings[ctf_name][key])
                )
        else:
            continue

    command.append('{0}'.format(file_input))

    return ' '.join(command)

def create_cter_1_0_command(
        ctf_name, file_input, output_dir, settings
        ):
    """Create the CTER v1.0 command"""

    try:
        shutil.rmtree(output_dir)
    except FileNotFoundError:
        pass

    command = []
    # Start the program
    command.append('{0}'.format(settings['Path'][ctf_name]))
    command.append("'{0}*'".format(file_input))
    command.append(output_dir)
    command.append('--selection_list')
    command.append(file_input)
    ignore_list = []
    ignore_list.append('Phase plate')
    if settings[ctf_name]['Phase plate'] == 'False':
        ignore_list.append('--defocus_min')
        ignore_list.append('--defocus_max')
        ignore_list.append('--defocus_step')
        ignore_list.append('--phase_min')
        ignore_list.append('--phase_max')
        ignore_list.append('--phase_step')
    else:
        command.append('--vpp')

    ignore_list.append('--pap')
    if settings[ctf_name]['--pap'] == 'True':
        command.append('--pap')
    else:
        pass

    for key in settings[ctf_name]:
        if key in ignore_list:
            continue
        elif settings[ctf_name][key]:
            command.append(key)
            command.append(
                '{0}'.format(settings[ctf_name][key])
                )
        else:
            continue

    return ' '.join(command)

def create_ctffind_4_v4_1_8_command(ctf_name, file_input, file_output, settings):
    """Create the ctffind command"""
    ctf_settings = settings[ctf_name]
    ctffind_command = []
    # Input file
    ctffind_command.append('{0}'.format(file_input))
    # Is movie
    if ctf_settings['Use movies'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
        ctffind_command.append('{0}'.format(ctf_settings['Combine frames']))
    else:
        pass
    # Output file
    ctffind_command.append('{0}'.format(file_output))
    # Pixel size
    ctffind_command.append('{0}'.format(ctf_settings['Pixel size']))
    # Acceleration voltage
    ctffind_command.append('{0}'.format(ctf_settings['Acceleration voltage']))
    # Spherical abberation
    ctffind_command.append('{0}'.format(ctf_settings['Spherical aberration']))
    # Amplitude contrast
    ctffind_command.append('{0}'.format(ctf_settings['Amplitude contrast']))
    # Amplitude spectrum
    ctffind_command.append('{0}'.format(ctf_settings['Amplitude spectrum']))
    # Minimum resolution
    ctffind_command.append('{0}'.format(ctf_settings['Min resolution(A)']))
    # Maximum resolution
    ctffind_command.append('{0}'.format(ctf_settings['Max resolution(A)']))
    # Minimum defocus
    ctffind_command.append('{0}'.format(ctf_settings['Min defocus(A)']))
    # Maximum defocus
    ctffind_command.append('{0}'.format(ctf_settings['Max defocus(A)']))
    # Defocus step
    ctffind_command.append('{0}'.format(ctf_settings['Step defocus(A)']))
    # Do you know astigmatism
    if ctf_settings['Know astigmatism'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Slow or fast
    if ctf_settings['High accuracy'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Astigmatism ctf_settings
    if ctf_settings['Know astigmatism'] == 'True':
        ctffind_command.append('{0}'.format(ctf_settings['Astigmatism']))
        ctffind_command.append('{0}'.format(ctf_settings['Astigmatism angle']))
    else:
        if ctf_settings['Restrain astigmatism'] == 'True':
            ctffind_command.append('{0}'.format('yes'))
            ctffind_command.append(
                '{0}'.format(ctf_settings['Expected astigmatism'])
                )
        else:
            ctffind_command.append('{0}'.format('no'))
    # Phase shift
    if ctf_settings['Phase shift'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
        ctffind_command.append('{0}'.format(ctf_settings['Min phase(rad)']))
        ctffind_command.append('{0}'.format(ctf_settings['Max phase(rad)']))
        ctffind_command.append('{0}'.format(ctf_settings['Step phase(rad)']))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Expert ctf_settings
    ctffind_command.append('{0}'.format('yes'))
    # Resmaple micrograph
    if ctf_settings['Resample micrographs'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Movie input ctf_settings
    if ctf_settings['Use movies'] == 'True':
        # Gain correction
        if ctf_settings['Movie is gain-corrected?'] == 'True':
            ctffind_command.append('{0}'.format('yes'))
        else:
            ctffind_command.append('{0}'.format('no'))
            ctffind_command.append('{0}'.format(ctf_settings['Gain file']))
        # Correct for magnifying distortions
        if ctf_settings['Correct mag. distort.'] == 'True':
            ctffind_command.append('{0}'.format('yes'))
            ctffind_command.append(
                '{0}'.format(ctf_settings['Mag. dist. angle'])
                )
            ctffind_command.append(
                '{0}'.format(ctf_settings['Mag. dist. major scale'])
                )
            ctffind_command.append(
                '{0}'.format(ctf_settings['Mag. dist. minor scale'])
                )
        else:
            ctffind_command.append('{0}'.format('no'))

    else:
        pass

    command = 'echo "{0}" | {1}'.format(
        '\n'.join(ctffind_command),
        settings['Path'][ctf_name]
        )
    return command


def combine_ctf_outputs(
        root_path,
        file_name,
        settings,
        queue_com,
        shared_dict,
        name,
        sum_file
        ):
    """
    Combine the ctf outputs to one SPHIRE partres and one RELION star file.

    root_path - Root path of the file
    file_name - File name of the ctf file.
    settings - TranSPHIRE settings
    queue_com - Queue for communication
    name - Name of process
    sum_file - Name of the dose uncorrected sum file

    Returns:
    None
    """

    ctf_name = settings['Copy']['CTF']
    ctf_settings = settings[ctf_name]
    ctf_folder = settings['ctf_folder']
    project_folder = '{0}/'.format(settings['project_folder'])

    data, data_orig = tu.import_ctf(ctf_name, ctf_folder)

    if ctf_name.lower().startswith('cter'):
        data_star = data
    else:
        data_star = data_orig

    lines = to_star_file(
        data=data_star,
        ctf_name=ctf_name,
        ctf_settings=ctf_settings,
        project_folder=project_folder,
        ctf_folder=ctf_folder,
        sum_file=sum_file
        )
    output_name_star = os.path.join(
        project_folder,
        '{0}_transphire.star'.format(ctf_name.replace(' ', '_'))
        )

    shared_dict['ctf_star_lock'].lock()
    try:
        with open(output_name_star, 'w') as write:
            write.write(lines)
    except Exception:
        raise
    finally:
        shared_dict['ctf_star_lock'].unlock()

    lines = to_partres_file(
        data=data_orig,
        ctf_name=ctf_name,
        ctf_settings=ctf_settings,
        project_folder=project_folder,
        ctf_folder=ctf_folder,
        sum_file=sum_file
        )
    output_name_partres = os.path.join(
        project_folder,
        '{0}_transphire_partres.txt'.format(ctf_name.replace(' ', '_'))
        )

    shared_dict['ctf_partres_lock'].lock()
    try:
        with open(output_name_partres, 'w') as write:
            write.write(lines)
    except Exception:
        raise
    finally:
        shared_dict['ctf_partres_lock'].unlock()

    return output_name_partres, output_name_star


def to_star_file(data, ctf_name, ctf_settings, project_folder, ctf_folder, sum_file):
    """
    Create a CTF star file from data

    data - Array containing ctf information.
    ctf_name - Name of the ctf program.
    ctf_settings - Settings for this ctf estimation run.
    project_folder - Name of the project folder.
    ctf_folder - Name of the ctf output folder.
    sum_file - Name of the sum file

    Returns:
    None
    """
    export_dtype = data.dtype.descr
    extension_dtype = [
        ('_rlnCtfImage', '|S200'),
        ('_rlnVoltage', '<f8'),
        ('_rlnSphericalAberration', '<f8'),
        ('_rlnAmplitudeContrast', '<f8'),
        ('_rlnMagnification', '<f8'),
        ('_rlnDetectorPixelSize', '<f8'),
        ]

    export_dtype.extend(extension_dtype)
    export_data = np.atleast_1d(np.empty(data.shape[0], dtype=export_dtype))
    header = get_relion_header(names=export_data.dtype.names)

    for idx, row in enumerate(data):

        for name in row.dtype.names:

            if name == 'file_name':
                value = sum_file.replace(project_folder, '').encode()
            elif name == 'defocus':
                value = (1 * row['defocus_diff'] + 2 * row['defocus']) / 2
            elif name == 'defocus_diff':
                value = 2 * row['defocus'] - export_data[idx]['defocus']
            else:
                value = row[name]
            export_data[idx][name] = value

        for key, _ in extension_dtype:
            value = get_constant_value(key, ctf_settings, row, project_folder, ctf_name, ctf_folder)
            export_data[idx][key] = value

    lines = [header]
    maximum_string = {
        '_rlnCtfImage': len(max(export_data['_rlnCtfImage'], key=len)),
        'file_name': len(max(export_data['file_name'], key=len))
        }
    create_export_data(
        export_data=export_data,
        lines=lines,
        maximum_string=maximum_string
        )
    return '\n'.join(lines)


def create_export_data(export_data, lines, maximum_string):
    """
    Write export data to file.

    export_data - Data to export.
    file_name - Name of the file to write to.

    Returns:
    In place modificaion of lines
    """
    for row in export_data:
        row_string = []
        for name in export_data.dtype.names:
            if name == 'mic_number':
                continue
            else:
                pass
            value = row[name]
            if isinstance(value, int):
                row_string.append('{0: 7d}'.format(value))
            elif isinstance(value, float):
                row_string.append('{0: 14f}'.format(value))
            else:
                length = maximum_string[name]
                row_string.append('{0:{1}s}'.format(value.decode('utf-8'), length))
        lines.append('\t'.join(row_string))


def to_partres_file(data, ctf_name, ctf_settings, project_folder, ctf_folder, sum_file):
    """
    Create a CTF partres file from data

    data - Array containing ctf information.
    ctf_name - Name of the ctf program.
    ctf_settings - Settings for this ctf estimation run.
    project_folder - Name of the project folder.
    ctf_folder - Name of the ctf output folder.
    sum_file - Name of the summed micrograph image

    Returns:
    None
    """

    export_dtype = ti.get_dtype_import_dict()['CTER v1.0']
    export_data = np.atleast_1d(np.empty(data.shape[0], dtype=export_dtype))
    constant_settings = set([
        'cs',
        'volt',
        'apix',
        'const_amplitude_contrast',
        ])

    for idx, row in enumerate(data):

        for name in export_data.dtype.names:

            try:
                value = row[name] 
            except ValueError:
                # Is not a CTER partres name
                assert name != 'phase_shift'
                assert name != 'file_name'
                if name in constant_settings:
                    value = get_constant_value(name, ctf_settings, row, project_folder, ctf_name, ctf_folder)
                    if name == 'const_amplitude_contrast':
                        value *= 100
                    else:
                        pass
                elif name == 'reserved_spot':
                    value = row['cross_corr']
                elif name == 'defocus':
                    value = (row['defocus_1'] + row['defocus_2']) / 20000
                elif name == 'astigmatism_amplitude':
                    value = np.abs(row['defocus_1'] - row['defocus_2']) / 10000
                elif name == 'astigmatism_angle':
                    value = 45 - row['astigmatism']
                elif name == 'limit_defocus_and_astigmatism':
                    if row['limit'] == 0:
                        value = -1
                    else:
                        value = 1 / row['limit']
                elif name == 'limit_pixel_error':
                    value = 1 / (get_constant_value('apix', ctf_settings, row, project_folder, ctf_name, ctf_folder) * 2)
                elif name == 'amplitude_contrast':
                    contrast = get_constant_value('const_amplitude_contrast', ctf_settings, row, project_folder, ctf_name, ctf_folder) * 100
                    contrast = contrast_to_shift(contrast)
                    phase_shift = row['phase_shift']
                    value = shift_to_contrast(contrast + phase_shift)
                else:
                    value = 0
            else:
                # Is a CTER partres name
                if name == 'file_name':
                    value = sum_file.replace(project_folder, '').encode()
                else:
                    pass

            export_data[idx][name] = value

    lines = []
    maximum_string = {
        'file_name': len(max(export_data['file_name'], key=len))
        }
    create_export_data(
        export_data=export_data,
        lines=lines,
        maximum_string=maximum_string
        )
    return '\n'.join(lines)


def get_relion_header(names):
    """
    Create a relion star file header.

    names - Header names as list

    Returns:
    header string
    """
    header = []
    header.append('')
    header.append('data_')
    header.append('')
    header.append('loop_')
    transphire_dict = ti.get_transphire_dict()
    index = 0
    for name in names:
        try:
            new_name = transphire_dict[name]
        except KeyError:
            if name == 'mic_number':
                continue
            else:
                new_name = name
        header.append('{0} #{1}'.format(new_name, index+1))
        index += 1
    return '\n'.join(header)


def get_constant_value(key, ctf_settings, row, project_folder, ctf_name, ctf_folder):
    if key == '_rlnCtfImage':
        _, extension = os.path.splitext(row['file_name'])
        if ctf_name.lower().startswith('gctf'):
            file_name = row['file_name'].replace(
                extension,
                '.ctf:mrc'.encode()
                )
            file_name = os.path.join(ctf_folder.encode(), os.path.basename(file_name))
        elif ctf_name.lower().startswith('ctffind'):
            file_name = row['file_name'].replace(
                extension,
                '.mrc:mrc'.encode()
                )
            file_name = os.path.join(ctf_folder.encode(), os.path.basename(file_name))
        elif ctf_name.lower().startswith('cter'):
            file_name = 'not_available_with_cter_option'.encode()
        else:
            raise IOError('Cannot find ctf_name! {0}'.format(
                ctf_name
                ))
        value = file_name.replace(project_folder.encode(), b'')
    elif key == '_rlnVoltage' or key == 'volt':
        if '--kV' in ctf_settings:
            value = ctf_settings['--kV']
        elif 'Acceleration voltage' in ctf_settings:
            value = ctf_settings['Acceleration voltage']
        elif '--voltage' in ctf_settings:
            value = ctf_settings['--voltage']
        else:
            raise IOError('Cannot find voltage key! {0}'.format(
                ctf_name
                ))
        value = float(value)
    elif key == '_rlnSphericalAberration' or key == 'cs':
        if 'Spherical aberration' in ctf_settings:
            value = ctf_settings['Spherical aberration']
        elif '--cs' in ctf_settings:
            value = ctf_settings['--cs']
        elif '--Cs' in ctf_settings:
            value = ctf_settings['--Cs']
        else:
            raise IOError('Cannot find cs key! {0}'.format(
                ctf_name
                ))
        value = float(value)
    elif key == '_rlnAmplitudeContrast' or key == 'const_amplitude_contrast':
        if '--ac' in ctf_settings:
            value = ctf_settings['--ac']
            value = float(value)
            if value > 1:
                value /= 100
            else:
                pass
        elif 'Amplitude contrast' in ctf_settings:
            value = ctf_settings['Amplitude contrast']
        else:
            raise IOError('Cannot find ac key! {0}'.format(
                ctf_name
                ))
        value = float(value)
    elif key == '_rlnMagnification':
        value = 10000.0
        value = float(value)
    elif key == '_rlnDetectorPixelSize' or key == 'apix':
        if 'Pixel size' in ctf_settings:
            value = ctf_settings['Pixel size']
        elif '--apix' in ctf_settings:
            value = ctf_settings['--apix']
        else:
            raise IOError('Cannot find apix key! {0}'.format(
                ctf_name
                ))
        value = float(value)
    else:
        raise IOError('Dont know {0}'.format(
            key
            ))
    return value


def contrast_to_shift(a_cont):
    """
    Transform amplitude contrast to phase shift.

    a_cont - Amplitude contrast value in percent

    Returns:
    Phase shift in degrees.
    """
    a_cont = float(a_cont)
    if a_cont == 100.0:
        value = 90.0
    elif a_cont == -100.0:
        value = 90.0
    elif a_cont < 0.0:
        value =  np.degrees(np.arctan(a_cont / np.sqrt(1.0e4 - a_cont**2))) + 180.0
    else:
        value = np.degrees(np.arctan(a_cont / np.sqrt(1.0e4 - a_cont**2)))
    return value


def shift_to_contrast(phase_shift):
    """
    Convert phase shift to amplitud contrast.

    phase_shift - Phase shift in degrees.

    Returns:
    Amplitude contrast in percent.
    """
    return np.tan(np.radians(phase_shift)) / \
        np.sqrt(1.0 + np.tan(np.radians(phase_shift))**2) * 100.0



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
try:
    from PyQt4.QtGui import QWidget, QHBoxLayout, QLabel, QPushButton
    from PyQt4.QtCore import pyqtSignal, pyqtSlot
except ImportError:
    from PyQt5.QtWidgets import QWidget, QHBoxLayout, QLabel, QPushButton
    from PyQt5.QtCore import pyqtSignal, pyqtSlot


class FrameWidget(QWidget):
    """
    FrameWidget widget.

    Inherits from:
    QWidget

    Buttons:
    Delete - Delete this instance of the variable.

    Signals:
    delete - Delete this instance of the variable (class instance).
    """
    delete = pyqtSignal(object)

    def __init__(self, first, last, dose_weight, default, parent=None):
        """
        Setup the layout for the widget

        Arguments:
        first - First frame
        last - Last frame
        dose_weight - Do dose weighting or not
        default - Is this instance default or not
        parent - Parent widget (default None)

        Return:
        None
        """
        super(FrameWidget, self).__init__(parent)

        # Global content
        self.first = int(first)
        self.last = int(last)
        self.dose_weight = dose_weight
        self.default = default

        # Label content
        label_first = QLabel('First: {0}'.format(self.first), self)
        label_last = QLabel('Last: {0}'.format(self.last), self)
        label_dose_weight = QLabel(
            'Dose weighting: {0}'.format(self.dose_weight),
            self
            )

        # Delete button
        if self.default:
            button = QPushButton('Default', self)
            button.setEnabled(False)
        else:
            button = QPushButton('Del', self)
            button.clicked.connect(self._button_clicked)

        # Add to layout
        layout = QHBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(label_first)
        layout.addWidget(label_last)
        layout.addWidget(label_dose_weight)
        layout.addWidget(button)

        # Object names
        label_first.setObjectName('entry')
        label_last.setObjectName('entry')
        label_dose_weight.setObjectName('entry')
        button.setObjectName('button_entry')

    @pyqtSlot()
    def _button_clicked(self):
        """
        Emit the signal with the information

        Arguments:
        None

        Return:
        None
        """
        self.delete.emit(self)

    def get_settings(self):
        """
        Get the settings

        Arguments:
        None

        Return:
        settings as dictionary
        """
        settings = {}
        settings['first'] = self.first
        settings['last'] = self.last
        settings['dw'] = self.dose_weight
        settings['default'] = self.default
        return settings



import setuptools
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name='transphire',
    version='1.0.0',
    description='Automated post data aquisition processing tool',
    long_description=long_description,
    url='https://github.com/mstabrin/transphire',
    author='Markus Stabrin',
    author_email='markus.stabrin@tu-dortmund.de',
    license='GPLv3',
    packages=setuptools.find_packages(exclude=[]),
    entry_points={
        'console_scripts': [
            'transphire = transphire.__main__:run_package'
            ]
        },
    install_requires = [
        'numpy>=1.14.0,<1.15.0',
        'matplotlib>=2.2.0,<2.3.0',
        'pexpect>=4.6.0,<4.7.0',
        'cython>=0.28.0,<0.29.0',
        'telepot>=12.0,<13.0',
        ],
    classifiers = (
        'Programming Language :: Python :: 3.6',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: POSIX :: Linux',
        'Environment :: X11 Applications :: Qt',
        'Development Status :: 4 - Beta'
        ),
    )




"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
import os
import copy
try:
    QT_VERSION = 4
    from PyQt4.QtGui import (
        QWidget,
        QVBoxLayout,
        QLabel,
        QPushButton,
        QLineEdit,
        QHBoxLayout,
        QComboBox,
        QFileDialog
        )
    from PyQt4.QtCore import pyqtSignal, pyqtSlot
except ImportError:
    QT_VERSION = 5
    from PyQt5.QtWidgets import (
        QWidget,
        QVBoxLayout,
        QLabel,
        QPushButton,
        QLineEdit,
        QHBoxLayout,
        QComboBox,
        QFileDialog
        )
    from PyQt5.QtCore import pyqtSignal, pyqtSlot
from transphire import transphire_utils as tu


class LoadContent(QWidget):
    """
    LoadContent widget.
    Widget used for the LoadContentContainer.

    Inherits from:
    QWidget

    Signals:
    delete - Emited, when the delete button is pressed (object)
    """
    delete = pyqtSignal(object)

    def __init__(self, typ, separator, settings_folder, hdd=None, parent=None):
        """
        Setup the layout for the widget

        Arguments:
        typ - Typ/Name of content
        separator - Separator widget list to delete if necessary
        hdd - Content is hdd (default None)
        parent - Parent widget (default None)

        Return:
        None
        """
        super(LoadContent, self).__init__(parent)
        # Variables
        self.separator = separator
        self.typ = typ
        self.layout = QVBoxLayout(self)
        self.content = []
        self.layout.setContentsMargins(0, 0, 0, 0)

        self.idx_name = 0
        self.idx_values = 1
        self.idx_dtype = 2
        self.idx_group = 3
        self.idx_type = 4

        # Fill content based on typ
        content_function = tu.get_function_dict()[typ]['content']
        if typ == 'Mount':
            button = QPushButton('Delete', self)
            button.clicked.connect(self._button_clicked)
            self.layout.addWidget(button)
            items = content_function(hdd=hdd)
            if hdd is None:
                pass
            else:
                self.setEnabled(False)
        elif typ == 'Copy':
            items = content_function(settings_folder=settings_folder)
        else:
            items = content_function()

        # Fill widget with content items
        self._fill_default(items)

    def _fill_default(self, items):
        """
        Fill the widget with content.

        Arguments:
        items - Content for the widget as list

        Return:
        None
        """
        # Layout
        layout_h = QHBoxLayout()
        self.layout.addLayout(layout_h)
        layout_v = None
        for idx, entry in enumerate(items):
            if idx % 10 == 0:
                if layout_v is not None:
                    layout_v.addStretch(1)
                else:
                    pass
                layout_v = QVBoxLayout()
                layout_h.addLayout(layout_v)
            layout_v.addWidget(QLabel(entry[self.idx_name], self))

            # Behaviour based on typ
            widget_2 = None
            if entry[self.idx_type] == 'COMBO':
                widget = QComboBox(self)
                widget.addItems(entry[self.idx_values])
                widget.setCurrentIndex(0)
                widget.currentIndexChanged.connect(self._change_color_to_changed)
            elif entry[self.idx_type] == 'DIR':
                widget = QLineEdit(entry[self.idx_values], self)
                widget.textChanged.connect(self._change_color_to_changed)
                widget.returnPressed.connect(self._find_dir)
                widget.setPlaceholderText('Press shift+return')
            elif entry[self.idx_type] == 'FILE':
                widget = QLineEdit(entry[self.idx_values], self)
                widget.textChanged.connect(self._change_color_to_changed)
                widget.returnPressed.connect(self._find_file)
                widget.setPlaceholderText('Press shift+return')
            elif entry[self.idx_type] == 'FILE/CHOICE':
                widget = QLineEdit(entry[self.idx_values], self)
                widget.textChanged.connect(self._change_color_to_changed)
                widget.returnPressed.connect(self._find_file)
                widget.setPlaceholderText('Press shift+return')
                widget_2 = QComboBox(self)
                widget_2.addItems(['True', 'False'])
                widget_2.setCurrentIndex(0)
                widget_2.currentIndexChanged.connect(self._change_color_to_changed)
            else:
                widget = QLineEdit(entry[self.idx_values], self)
                widget.textChanged.connect(self._change_color_to_changed)
            widget.setObjectName('default_settings')
            if widget_2 is None:
                layout_v.addWidget(widget, stretch=1)
            else:
                widget_2.setObjectName('default_settings')
                layout_h_2 = QHBoxLayout()
                layout_h_2.addWidget(widget, stretch=1)
                layout_h_2.addWidget(widget_2, stretch=0)
                layout_v.addLayout(layout_h_2)
            self.content.append({
                'widget': widget,
                'widget_2': widget_2,
                'settings': {
                    'typ': entry[self.idx_type],
                    'name': entry[self.idx_name],
                    'values': entry[self.idx_values],
                    'dtype': entry[self.idx_dtype],
                    'group': entry[self.idx_group],
                    }
                })
        layout_v.addStretch(1)

    @pyqtSlot()
    def _change_color_to_changed(self):
        """
        Change the color of the entry to color changed.

        Arguments:
        None

        Return:
        None
        """
        self.sender().setStyleSheet(tu.get_style(typ='changed'))

    @pyqtSlot()
    def _button_clicked(self):
        """
        Emit the signal, if the delete button is clicked.
        Send the seperator widget for deletion.

        Arguments:
        None

        Return:
        None
        """
        self.delete.emit(self.separator)

    def get_settings(self):
        """
        Get the settings from the child widgets.

        Arguments:
        None

        Return:
        List of settings
        """
        settings_list = []
        error_occured = False
        for entry in self.content:
            widget = entry['widget']
            widget_2 = entry['widget_2']
            settings = copy.deepcopy(entry['settings'])
            dtype = settings['dtype']
            key = settings['name']
            if isinstance(widget, QComboBox):
                value = widget.currentText()
            elif isinstance(widget, QLineEdit):
                value = widget.text()
            else:
                message = '{0}: Type {1} not known!'.format(key, type(widget))
                print(message)
                tu.message(message)
                sys.exit()

            if isinstance(widget_2, QComboBox):
                settings['widget_2'] = widget_2.currentText()
                widget_2.setStyleSheet(tu.get_style(typ='pass'))
            else:
                assert widget_2 is None
                settings['widget_2'] = widget_2

            if value:
                if tu.check_instance(value=value, typ=dtype):
                    widget.setStyleSheet(tu.get_style(typ='pass'))
                else:
                    widget.setStyleSheet(tu.get_style(typ='error'))
                    message = '{0}: {1} needs to be {2}'.format(
                        key,
                        value,
                        dtype
                        )
                    print(message)
                    tu.message(message)
                    error_occured = True
            else:
                pass

            if key == 'Typ' and not widget.isEnabled():
                value = 'Copy_hdd'
            elif key == 'Protocol' and not widget.isEnabled():
                value = 'hdd'
            else:
                pass
            settings_list.append({key: [value, settings]})

        if error_occured:
            return None
        else:
            return settings_list

    def set_settings(self, settings):
        """
        Set settings to the entry widgets.

        Arguments:
        settings - List of settings

        Return:
        None
        """
        disable_name = False
        disable_typ = False
        for entry in settings:
            if self.typ == 'Mount':
                if 'Mount name' in entry:
                    if entry['Mount name'][0] == 'HDD':
                        disable_name = True
                    else:
                        pass
                elif 'Typ' in entry:
                    if entry['Typ'][0] == 'Copy_hdd':
                        disable_typ = True
                    else:
                        pass
                else:
                    pass
            else:
                pass

            if disable_name and disable_typ:
                is_hdd = True
                self.setEnabled(False)
            else:
                is_hdd = False
                self.setEnabled(True)

            for key in entry:
                idx = None
                for idx, setting in enumerate(self.content):
                    name = setting['settings']['name']
                    if key != name:
                        continue
                    else:
                        break

                if idx is None:
                    continue
                else:
                    pass

                if name == key:
                    pass
                else:
                    continue

                try:
                    widget = self.content[idx]['widget']
                except KeyError:
                    continue
                else:
                    pass

                if isinstance(widget, QComboBox):
                    idx = widget.findText(entry[key][0])
                    if idx < 0:
                        if is_hdd:
                            widget.addItem('Copy_hdd')
                            idx = widget.findText('Copy_hdd')
                        else:
                            idx = 0
                    else:
                        pass
                    widget.setCurrentIndex(idx)
                elif isinstance(widget, QLineEdit):
                    widget.setText(entry[key][0])
                else:
                    print(key, type(widget, 'Not known!'))
                    sys.exit()
                widget.setStyleSheet(tu.get_style(typ='unchanged'))

                try:
                    widget_2 = self.content[idx]['widget_2']
                except KeyError:
                    continue
                else:
                    pass

                if isinstance(widget_2, QComboBox):
                    try:
                        idx = widget_2.findText(entry[key][1]['widget_2'])
                    except KeyError:
                        print(
                            'Older version of a save file detected!',
                            'Please save default settings again!'
                            )
                        continue
                    else:
                        widget_2.setCurrentIndex(idx)
                        widget_2.setStyleSheet(tu.get_style(typ='unchanged'))
                elif widget_2 is None:
                    pass
                else:
                    pass

    @pyqtSlot()
    def _find_file(self):
        """
        Find file name to insert it into the widget.

        Arguments:
        None

        Return:
        None
        """
        in_file = QFileDialog.getOpenFileName(
            caption='Find file: {0}'.format(self.typ),
            directory=os.getcwd(),
            options=QFileDialog.DontUseNativeDialog
            )

        if QT_VERSION == 4:
            in_file = in_file
        elif QT_VERSION == 5:
            in_file = in_file[0]
        else:
            raise ImportError('QT version unknown! Please contact the transphire authors!')

        if in_file != '':
            self.sender().setText(in_file)

    @pyqtSlot()
    def _find_dir(self):
        """
        Find directory to insert it into the widget.

        Arguments:
        None

        Return:
        None
        """
        in_dir = QFileDialog.getExistingDirectory(
            caption='Find directory: {0}'.format(self.typ),
            directory=os.getcwd(),
            options=QFileDialog.DontUseNativeDialog
            )
        if in_dir != '':
            self.sender().setText(in_dir)



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
try:
    from PyQt4.QtGui import QWidget, QVBoxLayout, QTabWidget
except ImportError:
    from PyQt5.QtWidgets import QWidget, QVBoxLayout, QTabWidget


class TabDocker(QWidget):
    """
    Tab widget for the settingswidgets.

    Inherits:
    QWidget
    """

    def __init__(self, parent=None, **kwargs):
        """
        Initialise layout for TabDocker

        Arguments:
        parent - Parent widget (default None)

        Return:
        None
        """
        super(TabDocker, self).__init__(parent)

        layout = QVBoxLayout(self)
        self.tab_widget = QTabWidget(self)
        self.tab_widget.setObjectName('tab')
        layout.addWidget(self.tab_widget)

    def add_tab(self, widget, name):
        """
        Add a new tab to the TabDocker

        Arguments:
        widget - Widget to add
        name - Name of the widget

        Return:
        None
        """
        self.tab_widget.addTab(widget, name)



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
import warnings
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt4agg import (
    FigureCanvasQTAgg as FigureCanvas,
    NavigationToolbar2QT as NavigationToolbar
    )
try:
    from PyQt4.QtGui import QWidget, QVBoxLayout
except ImportError:
    from PyQt5.QtWidgets import QWidget, QVBoxLayout
from transphire import transphire_utils as tu
warnings.filterwarnings('ignore')


class PlotWidget(QWidget):
    """
    PlotWidget widget.
    Widget used to show data plots.

    Inherits from:
    QWidget

    Signals:
    None
    """

    def __init__(self, label, plot_typ, *args, parent=None, **kwargs):
        """
        Setup the layout for the widget.

        Arguments:
        label - Label of the plot.
        plot_typ - Type of plot (e.g. histogram, values)
        parent - Parent widget (Default None)
        *args - Unused additional arguments
        **kwargs - Unused named additional arguments

        Return:
        None
        """
        super(PlotWidget, self).__init__(parent)

        # Variables
        self.label = label
        self.plot_typ = plot_typ

        # Initialise figure
        self.figure = plt.figure()
        self.compute_initial_figure()

        # Create canvas for the figure
        self.canvas = FigureCanvas(self.figure)
        self.canvas.setParent(self)
        self.toolbar = NavigationToolbar(self.canvas, self)
        self.canvas.draw()

        # Fill layout
        layout_v = QVBoxLayout(self)
        layout_v.addWidget(self.toolbar)
        layout_v.addWidget(self.canvas)

    def compute_initial_figure(self):
        """
        Compute the initial figure shown on startup.

        Arguments:
        None

        Return:
        None
        """
        self.figure.clear()
        ax1 = self.figure.add_subplot(111)
        ax1.plot([0, 0, 0, 0.5, 0.5, 0.5], [0, 1, 0.5, 0.5, 0, 1], 'b')
        ax1.plot([1.5, 1, 1, 1.5, 1, 1, 1.5], [0, 0, 0.5, 0.5, 0.5, 1, 1], 'b')
        ax1.plot([2, 2, 2.5], [1, 0, 0], 'b')
        ax1.plot([3, 3, 3.5], [1, 0, 0], 'b')
        ax1.plot([4, 4.15, 4.35, 4.5, 4.5, 4.35, 4.15, 4, 4], [0.3, 0, 0, 0.3, 0.6, 1, 1, 0.6, 0.3], 'b')

    def update_figure(self, name, data, directory_name, settings):
        """
        Update the figure with data plot.

        Arguments:
        name - Name of the plot type
        data - Data to plot
        directory_name - Name of the output directory
        settings - Settings provided by the user

        Return:
        None
        """
        if name == 'False':
            return None
        elif name == 'Later':
            return None
        else:
            x_values, y_values, label, title = tu.get_function_dict()[name]['plot'](
                data=data,
                settings=settings,
                label=self.label
                )

        self.figure.clear()
        ax1 = self.figure.add_subplot(111)

        if self.plot_typ == 'values':
            x_label = 'Micrograph ID'
            y_label = label
            ax1.plot(x_values, y_values, '.')
        elif self.plot_typ == 'histogram':
            x_label = label
            y_label = 'Nr. of micrographs'
            if np.max(y_values) - np.min(y_values) < 0.001:
                ax1.hist(y_values, 1)
                ax1.set_xlim([np.max(y_values)-1, np.min(y_values)+1])
            else:
                ax1.hist(y_values, 100)
        else:
            print('Unknown!')
            sys.exit()

        ax1.grid()
        ax1.set_title(title)
        ax1.set_xlabel(x_label)
        ax1.set_ylabel(y_label)
        self.canvas.draw()

        output_name = '{0}/{1}_{2}.png'.format(
            directory_name,
            self.label,
            self.plot_typ
            )
        try:
            self.figure.savefig(output_name.replace(' ', '_'))
        except RuntimeError:
            pass
        except FileNotFoundError:
            pass



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import json
import copy
try:
    from PyQt4.QtGui import QWidget, QPushButton, QVBoxLayout, QHBoxLayout, QTabWidget
    from PyQt4.QtCore import pyqtSlot
except ImportError:
    from PyQt5.QtWidgets import QWidget, QPushButton, QVBoxLayout, QHBoxLayout, QTabWidget
    from PyQt5.QtCore import pyqtSlot

from transphire.loadcontent import LoadContent
from transphire.separator import Separator
from transphire import transphire_utils as tu


class LoadContentContainer(QWidget):
    """
    LoadContentContainer widget
    Inherits from:
    QWidget

    Buttons:
    Save settings - Save currently defined settings to settings files
    Add - Add new Mount entry (Case Mount)
    """

    def __init__(self, typ, file_name, settings_folder, parent=None):
        """
        Setup the layout for the widget

        Arguments:
        typ - Typ/Name of content container
        file_name - File name for the saving settings
        parent - Parent widget (default None)

        Return:
        None
        """
        super(LoadContentContainer, self).__init__(parent)

        # Variables
        self.layout = QHBoxLayout()
        self.content = []
        self.file = file_name
        self.settings_folder = settings_folder
        self.typ = typ
        self.name = '{0} settings'.format(self.typ)
        self.tab_widget = None

        # Layout
        layout = QVBoxLayout(self)

        # Save button
        save_button = QPushButton('Save settings', self)
        save_button.clicked.connect(self.save_settings)

        # Typ based settings
        if self.typ == 'Mount':
            self.tab_widget = QTabWidget(self)
            self.tab_widget.setMovable(True)
            self.layout.addWidget(self.tab_widget)
            add_button = QPushButton(self)
            add_button.clicked.connect(self.add_widget)
            layout.addWidget(add_button)
            layout.addWidget(Separator(typ='horizontal', color='black'))
            add_button.setText('Add mount point')
            hdd = True
        else:
            hdd = None

        # Add HDD entry
        self.add_widget(name='HDD', hdd=hdd)
        self.setWindowTitle(self.name)

        # Add layout to global layout
        layout.addLayout(self.layout)
        layout.addStretch(1)
        layout.addWidget(Separator(typ='horizontal', color='black'))
        layout.addWidget(save_button)

        # Set minimum width of widget
        title_length = len(self.windowTitle())
        self.setMinimumWidth(title_length * 21)

    @pyqtSlot()
    def add_widget(self, name=None, hdd=None):
        """
        Add new widget to layout.

        Arguments:
        name - Name of new mount point (default None)
        hdd - Is hdd entry (default None)

        Return:
        None
        """
        # Widget for entry
        separator_1 = Separator(typ='vertical', color='white')
        separator_2 = Separator(typ='vertical', color='white')
        widget = LoadContent(
            typ=self.typ,
            hdd=hdd,
            parent=self,
            separator=[separator_1, separator_2],
            settings_folder=self.settings_folder
            )
        widget.delete.connect(self.remove_widget)
        self.content.append(widget)

        # If tab_widget exists, add to tab_widget, else to layout
        if self.tab_widget is not None:
            if name is None:
                self.tab_widget.addTab(
                    widget,
                    'Mount {0}'.format(self.tab_widget.count() + 1)
                    )
            else:
                self.tab_widget.addTab(widget, name)
        else:
            self.layout.addWidget(separator_1)
            self.layout.addWidget(widget)
            self.layout.addWidget(separator_2)

    @pyqtSlot(object)
    def remove_widget(self, separator):
        """
        Remove widget from layout.

        Arguments:
        separator - List of seperator widgets

        Return:
        None
        """
        # Sending widget
        widget = self.sender()

        # If tab widget exists, remove from tab widget, else from layout
        if self.tab_widget is not None:
            idx = self.tab_widget.indexOf(widget)
            self.tab_widget.removeTab(idx)

            # Rename mount entry widgets
            for idx in range(self.tab_widget.count()):
                if self.tab_widget.tabText(idx).startswith('Mount '):
                    self.tab_widget.setTabText(idx, 'Mount {0}'.format(idx + 1))
                else:
                    pass
        else:
            self.layout.removeWidget(widget)

        # Remove from content
        self.content.remove(widget)
        widget.setParent(None)
        for entry in separator:
            self.layout.removeWidget(entry)
            entry.setParent(None)

    def save_settings(self):
        """
        Save settings specified in widget to json text file.

        Arguments:
        None

        Return:
        None
        """
        settings = copy.deepcopy(self.get_settings())

        if settings is None:
            return None
        path = self.file.split('/')[:-1]
        tu.mkdir_p('/'.join(path))

        for entry in settings:
            for widget in entry:
                for key in widget:
                    widget[key][1].pop('dtype')

        with open(self.file, 'w') as file_w:
            json.dump(settings, file_w)

        tu.message('{0} saved!'.format(self.name))

    def get_settings_tab(self):
        """
        Get settings from tab widget.

        Arguments:
        None

        Return:
        List of widget settings
        """
        settings = []
        for idx in range(self.tab_widget.count()):
            widget = self.tab_widget.widget(idx)
            entry_settings = widget.get_settings()
            if entry_settings is None:
                return None
            else:
                settings.append(entry_settings)
        return settings

    def get_settings_widget(self):
        """
        Get settings from normal widget.

        Arguments:
        None

        Return:
        List of widget settings
        """
        settings = []
        for entry in self.content:
            entry_settings = entry.get_settings()
            if entry_settings is None:
                return None
            else:
                settings.append(entry_settings)
        return settings

    def get_settings(self):
        """
        Get settings from tab or normal normal widget.

        Arguments:
        None

        Return:
        List of widget settings
        """
        if self.tab_widget is None:
            settings = self.get_settings_widget()
        else:
            settings = self.get_settings_tab()

        return settings

    def set_settings(self, settings):
        """
        Set settings to widget.

        Arguments:
        settings - List of widget settings

        Return:
        None
        """
        for idx, entry in enumerate(settings):
            if self.typ == 'Mount':
                name = entry[0]['Mount name'][0]
                if idx != 0:
                    self.add_widget(name=name)
                else:
                    self.tab_widget.setTabText(idx, name)
            else:
                pass
            self.content[idx].set_settings(entry)



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
try:
    from PyQt4.QtGui import QWidget, QHBoxLayout, QPushButton, QInputDialog, QLineEdit
    from PyQt4.QtCore import pyqtSlot
except ImportError:
    from PyQt5.QtWidgets import QWidget, QHBoxLayout, QPushButton, QInputDialog, QLineEdit
    from PyQt5.QtCore import pyqtSlot
from transphire.passworddialog import PasswordDialog
from transphire import transphire_utils as tu


class MountWidget(QWidget):
    """
    MountWidget widget.

    Inherits from:
    QWidget

    Buttons:
    mount_button - Mount mount point
    umount_button - Umount mount point

    Signals:
    None
    """

    def __init__(self, content, mount_worker, parent=None):
        """
        Setup the layout for the widget

        Arguments:
        content - Mount content
        mount_worker - Worker thread instance.
        parent - Parent widget (default None)

        Return:
        None
        """
        super(MountWidget, self).__init__(parent)

        # Global content
        self.content = content
        self.name = content['Mount name'][0]
        self.ip_adress = content['IP'][0]
        self.typ = content['Typ'][0]
        self.default_user = content['Default user'][0]
        self.sec = content['sec'][0]
        self.gid = content['gid'][0]
        self.domain = content['Domain'][0]
        self.protocol = content['Protocol'][0]
        self.folder = content['Folder'][0]
        self.extension = bool(content['Need folder extension?'][0] == 'True')
        self.version = content['Protocol version'][0]
        self.ssh_address = content['SSH address'][0]
        self.quota_command = content['Quota command'][0]
        self.right_quota = content['Is df giving the right quota?'][0]
        self.quota = content['Quota / TB'][0]
        self.login = content['Target UID exists here and on target?'][0]
        self.mount_folder = self.name.replace(' ', '_')
        self.mount_worker = mount_worker
        self.thread_object = None

        # Content
        self.mount_button = QPushButton('{0}'.format(self.name), self)
        self.mount_button.setObjectName('mount')
        self.umount_button = QPushButton('{0}'.format(self.name), self)
        self.umount_button.setObjectName('mount')

        # Layout
        layout = QHBoxLayout(self)
        layout.addWidget(self.mount_button)
        layout.addWidget(self.umount_button)
        layout.setContentsMargins(0, 0, 0, 0)

        # events
        self.mount_button.clicked.connect(self.mount)
        self.umount_button.clicked.connect(self.umount)

    def set_thread_object(self, thread_object):
        """
        Set the thread object.

        thread_object - Thread object to set.

        Returns:
        None
        """
        self.thread_object = thread_object

    def get_settings(self):
        """
        Return settings of the content.

        Arguments:
        None

        Returns:
        Settings as dictionary
        """
        settings = {}
        for key in self.content:
            settings[key] = self.content[key][0]
        return settings

    @pyqtSlot()
    def mount(self):
        """
        Mount device preparation

        Arguments:
        None

        Return:
        None
        """
        # Mount external hdd
        if self.typ == 'Copy_hdd':
            self.mount_worker.sig_mount_hdd.emit(self.mount_folder)
        else:
            user, password, folder = self._check_user(
                login=bool(self.login == 'True')
                )
            if user:
                self.mount_worker.sig_mount.emit(
                    self.mount_folder,
                    user,
                    password,
                    folder,
                    self.ip_adress,
                    self.protocol,
                    self.domain,
                    self.version,
                    self.sec,
                    self.gid,
                    )
            else:
                print('Wrong identity')
                tu.message('Wrong identity, check command line for more information')

    @pyqtSlot()
    def umount(self):
        """
        Umount device signal preparation

        Arguments:
        None

        Return:
        None
        """
        if self.typ == 'Copy_hdd':
            hdd_folder = '{0}/{1}'.format(
                self.mount_worker.mount_directory,
                self.mount_folder
                )
            if not os.path.exists(hdd_folder):
                self.mount_worker.sig_info.emit(
                    'First mount {0}'.format(self.mount_folder)
                    )
                return None
            else:
                pass

            folders = os.listdir(hdd_folder)
            hdd_name = self._specify_hdd()
            if hdd_name is None:
                return None
            else:
                hdd_name = hdd_name.replace(' ', '_').upper()

            if hdd_name == 'ALL':
                for entry in folders:
                    self.mount_worker.sig_umount.emit(
                        self.mount_folder,
                        entry,
                        self.thread_object
                        )
            elif hdd_name.startswith('HDD_'):
                if hdd_name in folders:
                    self.mount_worker.sig_umount.emit(
                        self.mount_folder,
                        hdd_name,
                        self.thread_object
                        )
                else:
                    tu.message('{0} is not mounted!'.format(hdd_name))
            else:
                print('Unmount hdd: Unreachable code')
                print(hdd_name, self.mount_folder, folders)

        else:
            self.mount_worker.sig_umount.emit(
                self.mount_folder,
                self.mount_folder,
                self.thread_object
                )

    def _check_user(self, login):
        """
        Check if the user is allowed to mount

        Arguments:
        login - Try to login as user on this machine

        Return:
        Username, User password, Mount folder
        """
        dialog = PasswordDialog(
            folder=self.folder,
            default=self.default_user,
            login=login,
            extension=self.extension,
            parent=self
            )
        dialog.exec_()
        if dialog.result():
            return dialog.username, dialog.password, dialog.folder
        else:
            return False, False, False

    def _specify_hdd(self):
        """
        Specify which HDD to unmount.

        Arguments:
        None

        Return:
        HDD text
        """
        text, result = QInputDialog().getText(
            self,
            'Specify hdd',
            'choose hdd to unmount,\n"all" will unmount all mounted hdd\'s',
            QLineEdit.Normal,
            'all'
            )
        if result:
            text = text.lower()
            if text.startswith('hdd_'):
                return text
            elif text.startswith('hdd '):
                return text
            elif text == 'all':
                return text
            else:
                tu.message('Input needs to be "all" or "hdd_{number}" or "hdd {number}')
                return None
        else:
            tu.message('Input needs to be "all" or "hdd_{number} or "hdd {number}"')
            return None



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
try:
    from PyQt4.QtGui import QWidget, QVBoxLayout, QHBoxLayout, QLabel
except ImportError:
    from PyQt5.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QLabel
from transphire.mountwidget import MountWidget


class MountContainer(QWidget):
    """
    MountContainer widget.

    Inherits from:
    QWidget

    Buttons:
    None

    Signals:
    None
    """

    def __init__(self, content_mount, mount_worker, parent=None, **kwargs):
        """
        Setup the layout for the widget

        Arguments:
        content_mount - Mount points as transphire settings dict.
        mount_worker - Worker thread instance.
        parent - Parent widget (default None)
        kwargs - Unused arguments for easier automatisation

        Return:
        None
        """
        super(MountContainer, self).__init__(parent)

        # Global content
        self.content = {}

        # Layout
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)

        layout_label = QHBoxLayout()
        layout_label.addWidget(QLabel('Mount', self))
        layout_label.addWidget(QLabel('Umount', self))
        layout_label.setContentsMargins(0, 0, 0, 0)
        layout.addLayout(layout_label)

        # Temp content
        self.content_temp = []
        for entry in content_mount:
            content = {}
            for widget in entry:
                for key in widget:
                    content[key] = widget[key]
            self.content_temp.append(content)

        # Content
        for entry in self.content_temp:
            key_name = entry['Mount name'][0]
            if key_name:
                self.content[key_name] = MountWidget(
                    content=entry,
                    mount_worker=mount_worker,
                    parent=self
                    )
                layout.addWidget(self.content[key_name])

        layout.addStretch(1)

    def get_settings(self):
        """
        Return settings of the container.

        Arguments:
        None

        Return:
        Settings as list
        """
        settings = {}
        for key in self.content:
            settings[key] = self.content[key].get_settings()
        return [settings]

    def set_threadlist(self, thread_list):
        """
        Set the thread instances for the different objects.

        Arguments:
        thread_list - List of mount threads

        Return:
        None
        """
        for key in self.content:
            self.content[key].set_thread_object(
                thread_object=thread_list[key]['object']
                )

    def enable(self, var, use_all):
        """
        Enable or disable the content.

        Arguments:
        var - State of buttons (True or False)
        use_all - Disable all buttons (True) or only some (False)

        Return:
        None
        """
        for key in self.content:
            if use_all:
                self.content[key].setEnabled(var)
            else:
                self.content[key].setEnabled(var)
